<?php
declare(strict_types=1);

namespace Inti\Wp\Helpers\Provider;

use function is_string;

/**
 * Class ProviderList
 *
 * @package Inti\Wp\Helpers\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-helpers
 */
final class ProviderList implements ProviderListInterface
{
    private ProviderManager $manager;
    private array $providers;

    public function __construct(ProviderManager $manager, array $providers)
    {
        $this->manager = $manager;
        $this->providers = $providers;
    }

    public function canProvide(string $name, ?string $type = null): bool
    {
        return $this->getProvider($name, $type) instanceof ProviderInterface;
    }

    public function getProvider(string $name, ?string $type = null): ?ProviderInterface
    {
        foreach ($this->getProviders($type) as &$provider) {
            if ($provider->canProvide($name)) {
                return $provider;
            }
        }

        return null;
    }

    public function getProviders(?string $type = null): array
    {
        foreach ($this->providers as &$provider) {
            if (is_string($provider)) {
                $provider = $this->manager->get($provider);
            }

            if (isset($type)) {
                $providers[$type] = $provider;
            }
        }

        return isset($type)
            ? ($providers[$type] ?? [])
            : $this->providers;
    }
}
